"Droplet Helper Funs"
import subprocess
from contextlib import contextmanager
from os import chdir, getcwd
from os.path import dirname, expanduser, expandvars, join
from sys import executable
from tempfile import TemporaryDirectory

from digitalocean import Droplet, Manager

ANSIBLE = join(dirname(executable), "ansible-playbook")


class DropletNotFound(Exception):
    "Droplet Not Found Error"


def expandall(string):
    "Expand user home and env vars in string"
    return expanduser(expandvars(string))


def find_droplet_by_name(droplet_name: str, access_token: str) -> Droplet:
    """
    Find droplet by name.  Finds the first one by name, I think from
    any project.

    :param droplet_name: The name of the droplet to find.
    :param manager: A Manager object to use.

    """
    manager = Manager(token=access_token)

    try:
        return next(
            droplet
            for droplet in manager.get_all_droplets()
            if droplet.name == droplet_name
        )
    except StopIteration:
        raise DropletNotFound(f"Droplet {droplet_name} not found.") from None


def rsync(source, dest):
    "Runs rsync in a subprocess"
    subprocess.run(
        f"rsync -ruv '{source}' '{dest}' --exclude '*.part'", check=True, shell=True
    )


def ansible_playbook(inventory_file, playbook_path):
    "Call ansible-playbook with inventory and playbook files."
    subprocess.run(
        f"{ANSIBLE} -i {inventory_file} -u root {playbook_path}",
        shell=True,
        check=True,
    )


@contextmanager
def in_tempdir():
    "Change to a temporary directory and then back again"
    go_back_to = getcwd()
    with TemporaryDirectory() as tempdir:
        chdir(tempdir)
        try:
            yield tempdir
        finally:
            chdir(go_back_to)



"Download files from server."
import sys

import click
from click_conf_file import conf_option
from .constants import ACCESS_TOKEN
from .droplet_helper import DropletNotFound, find_droplet_by_name, rsync, expandall


@click.command("retrieve")
@click.option("-d", "--download_to", default="~/Downloads")
@click.option("-a", "--ip-address")
@click.option("-n", "--droplet-name", default="downloader")
@click.option("-t", "--access-token", type=str, envvar=ACCESS_TOKEN, show_envvar=True)
@conf_option(app_name="torrentdoer")
def retrieve_files(download_to, ip_address, droplet_name, access_token):
    "Download files from dest with rsync"
    if ip_address is None:
        try:
            ip_address = find_droplet_by_name(droplet_name, access_token).ip_address
            click.secho(f"Droplet: {ip_address}", fg="magenta", err=True)
        except DropletNotFound:
            click.secho("Does the server exist?", fg="red", err=True)
            sys.exit(1)

    rsync(f"root@{ip_address}:/var/lib/transmission/Downloads", expandall(download_to))



"ssh functions"
import socket
import time
from contextlib import closing
from subprocess import run

import click


def socket_available(host, port) -> bool:
    "Returns whether or not the socket can be opened."
    with closing(socket.socket(socket.AF_INET, socket.SOCK_STREAM)) as sock:
        return sock.connect_ex((host, port)) == 0


def make_tunnel(host, port, user="root"):
    "Create an SSH tunnel by shelling out to ssh."
    if socket_available("localhost", port):
        click.secho(f"Found port {port} already open.", fg="blue", err=True)
    else:
        click.secho(f"Setting up tunnel to {host}:{port}", fg="blue", err=True)
        run(f"ssh -L {port}:localhost:{port} -fN {user}@{host}", check=True, shell=True)


def wait_for_ssh(host):
    "Waits for SSH on host to be available."
    click.secho("Waiting for ssh to be available", err=True, fg="yellow", nl="")
    while not socket_available(host, 22):
        click.secho(".", err=True, fg="yellow", nl="")
        time.sleep(1)



"entrypoint module.  Sets up cli entrypoint."
import click
from .create import create_downloader
from .destroy import destroy_droplet
from .retrieve import retrieve_files
from .sshconnect import ssh_connect, ssh_tunnel


@click.group("torrent-doer")
def cli():
    """
    torrent-doer, creates and runs Transmission Daemon servers
    on a Digital Ocean Droplet.
    """


cli.add_command(create_downloader, "create")
cli.add_command(destroy_droplet, "destroy")
cli.add_command(retrieve_files, "retrieve")
cli.add_command(ssh_connect, "ssh")
cli.add_command(ssh_tunnel, "tunnel")



"Create Droplet"
import subprocess
import time
import toml

import click
import digitalocean
from click_conf_file import conf_option

from .constants import ACCESS_TOKEN
from .droplet_helper import (
    find_droplet_by_name,
    DropletNotFound,
    ansible_playbook,
    in_tempdir,
    expandall,
)
from .ssh import wait_for_ssh, make_tunnel


@click.command("create")
@click.option("-n", "--droplet-name", default="downloader")
@click.option("-r", "--region", default="sfo2")
@click.option("-s", "--size-slug", default="s-1vcpu-1gb")
@click.option("-c", "--connect", type=bool, default=False)
@click.option("-T", "--tunnel", type=bool, default=True)
@click.option("-i", "--image", default="centos-8-x64")
@click.option("-t", "--access-token", type=str, envvar=ACCESS_TOKEN, show_envvar=True)
@click.option("-p", "--transmission-port", default=9091)
@conf_option(app_name="torrentdoer", send_param=True)
def create_downloader(  # pylint: disable=R0913
    droplet_name,
    region,
    size_slug,
    connect,
    access_token,
    image,
    tunnel,
    transmission_port,
    config,
):
    "Create Downloader."
    try:
        droplet = find_droplet_by_name(droplet_name, access_token)
        click.secho(
            f"Found droplet at {droplet.ip_address}",
            fg="magenta",
            err=True,
        )
    except DropletNotFound:
        droplet = create_droplet(droplet_name, region, size_slug, access_token, image)
        wait_for_ssh(droplet.ip_address)
    configure_droplet_ansible(droplet.ip_address)
    if tunnel:
        make_tunnel(droplet.ip_address, transmission_port)
    if connect:
        subprocess.run("ssh root@" + droplet.ip_address, shell=True, check=True)
    # click.secho("\nssh root@" + droplet.ip_address, fg="magenta")
    config["ip_address"] = droplet.ip_address
    with open(expandall("~/.torrentdoerrc"), "w") as rcfile:
        toml.dump(config, rcfile)
    click.secho("Done.", fg="magenta", err=True)


def create_droplet(name, region, size_slug, access_token, image):
    "Makes a new droplet if one doesn't already exist."
    manager = digitalocean.Manager(token=access_token)
    keys = manager.get_all_sshkeys()
    click.secho("Creating droplet", fg="cyan")
    image = manager.get_image(image)
    droplet = digitalocean.Droplet(
        token=access_token,
        name=name,
        region=region,
        image=image.slug,
        size_slug=size_slug,
        ssh_keys=keys,
        backups=False,
    )
    droplet.create()
    while droplet.status != "active":
        time.sleep(1)
        click.secho(".", fg="yellow", nl="", err=True)
        droplet = manager.get_droplet(droplet.id)
    click.secho("", err=True)
    return droplet


def configure_droplet_ansible(ip_address):
    "Execute ansible"

    with in_tempdir():
        with open("hosts.cfg", "w") as hosts:
            hosts.write(f"[downloaders]\n{ip_address}\n")
        with open("playbook.yml", "w") as playbook:
            playbook.write(PLAYBOOK)
        with open("settings.json", "w") as settings_json:
            settings_json.write(SETTINGS_JSON)
        ansible_playbook("hosts.cfg", "playbook.yml")


PLAYBOOK = """
- hosts: downloaders
  tasks:
    - name: Install epel
      dnf:
        name:
          - epel-release
        state: latest
    - name: Install transmission-daemon and tmux
      dnf:
        name:
          - tmux
          - transmission-daemon
        state: latest
    - name: Set up Transmission Daemon
      systemd:
        name: transmission-daemon
        state: started
        enabled: yes
    - name: Ensure Directories Exist
      file:
        path: /var/lib/transmission/{item}
        state: directory
        mode: '0755'
        owner: transmission
      loop:
        - Downloads
        - ActiveDownloads
      notify:
        - Restart Transmission
    - name: Copy Settings.json
      copy:
        src: settings.json
        dest: /var/lib/transmission/.config/transmission-daemon/settings.json
        owner: transmission
        mode: '0644'
        force: no
      notify:
        - Restart Transmission
  handlers:
    - name: Restart Transmission
      systemd:
        name: transmission-daemon
        state: restarted
  remote_user: root
"""


SETTINGS_JSON = """
{
    "alt-speed-down": 50,
    "alt-speed-enabled": false,
    "alt-speed-time-begin": 540,
    "alt-speed-time-day": 127,
    "alt-speed-time-enabled": false,
    "alt-speed-time-end": 1020,
    "alt-speed-up": 50,
    "bind-address-ipv4": "0.0.0.0",
    "bind-address-ipv6": "::",
    "blocklist-enabled": false,
    "blocklist-url": "http://www.example.com/blocklist",
    "cache-size-mb": 4,
    "dht-enabled": true,
    "download-dir": "/var/lib/transmission/Downloads",
    "download-queue-enabled": true,
    "download-queue-size": 5,
    "encryption": 1,
    "idle-seeding-limit": 30,
    "idle-seeding-limit-enabled": false,
    "incomplete-dir": "/var/lib/transmission/ActiveDownloads",
    "incomplete-dir-enabled": false,
    "lpd-enabled": false,
    "message-level": 1,
    "peer-congestion-algorithm": "",
    "peer-id-ttl-hours": 6,
    "peer-limit-global": 200,
    "peer-limit-per-torrent": 50,
    "peer-port": 51413,
    "peer-port-random-high": 65535,
    "peer-port-random-low": 49152,
    "peer-port-random-on-start": false,
    "peer-socket-tos": "default",
    "pex-enabled": true,
    "port-forwarding-enabled": true,
    "preallocation": 1,
    "prefetch-enabled": true,
    "queue-stalled-enabled": true,
    "queue-stalled-minutes": 30,
    "ratio-limit": 2,
    "ratio-limit-enabled": false,
    "rename-partial-files": true,
    "rpc-authentication-required": false,
    "rpc-bind-address": "0.0.0.0",
    "rpc-enabled": true,
    "rpc-host-whitelist": "",
    "rpc-host-whitelist-enabled": true,
    "rpc-password": "{e46622e81d6587df0cf3dfc64cb5b236a52cdcce01mb8oZr",
    "rpc-port": 9091,
    "rpc-url": "/transmission/",
    "rpc-username": "",
    "rpc-whitelist": "127.0.0.1,::1",
    "rpc-whitelist-enabled": true,
    "scrape-paused-torrents-enabled": true,
    "script-torrent-done-enabled": false,
    "script-torrent-done-filename": "",
    "seed-queue-enabled": false,
    "seed-queue-size": 10,
    "speed-limit-down": 100,
    "speed-limit-down-enabled": false,
    "speed-limit-up": 100,
    "speed-limit-up-enabled": false,
    "start-added-torrents": true,
    "trash-original-torrent-files": false,
    "umask": 18,
    "upload-slots-per-torrent": 14,
    "utp-enabled": true
}

"""



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['torrentdoer']

package_data = \
{'': ['*']}

install_requires = \
['ansible>=2.10.3,<3.0.0',
 'click-conf-file>=0.1.2,<0.2.0',
 'click>=7.1.2,<8.0.0']

entry_points = \
{'console_scripts': ['torrentdoer = torrentdoer.entrypoint:cli']}

setup_kwargs = {
    'name': 'torrentdoer',
    'version': '0.2.0',
    'description': 'Do your torrenting from DigitalOcean.',
    'long_description': "# torrentdoer\n\nDo your torrenting from DigitalOcean!\n\nSets up a DigitalOcean droplet in your account,\nenables transmission-daemin in systemd, and opens up\nan SSH tunnel from port 9091 on your machine to the\nremote, for easy control.\n\n## Installation\n\n```\npython -m pip install torrentdoer\n```\n\n## Usage\n\nJust make sure you have an API Access Token from DigitalOcean.\nyou can either export `$DIGITALOCEAN_ACCESS_TOKEN` or else\npass the token to the `-t/--access-token` command-line option.\n\n### Start Server\n\n```bash\nexport DIGITALOCEAN_ACCESS_TOKEN='somuchsecret'\n\ntorrentdoer create\n```\n\nNow go ahead and open up a Transmission Client, pointing it to\nlocalhost:9091.\n\n### Retrieve Files\n\nThis will run rsync between the transmission daemon and your local machine.\n\n```bash\ntorrentdoer retrieve\n```\n\n### Remove The Server\n\nSave money by deleting the server when not in use.\n\n```bash\ntorrentdoer destroy\n```\n\n### Open SSH Session To Droplet\n\n\n```bash\ntorrentdoer ssh\n```\n\n### (Re-)Open SSH Tunnel\n\n\n```bash\ntorrentdoer tunnel\n```\n\nHave fun!\n",
    'author': 'Tyler Gannon',
    'author_email': 'tgannon@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/tylergannon/torrentdoer',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



"ssh functions"
import sys
from subprocess import run

from click_conf_file import conf_option
import click

from .constants import ACCESS_TOKEN
from .droplet_helper import find_droplet_by_name, DropletNotFound
from .ssh import make_tunnel


@click.command("ssh")
@click.option("-a", "--ip-address")
@click.option("-n", "--droplet-name", default="downloader")
@click.option("-T", "--access-token", type=str, envvar=ACCESS_TOKEN, show_envvar=True)
# @click.option("-p", "--transmission-port", default=9091)
@conf_option(app_name="torrentdoer")
def ssh_connect(  # pylint: disable=R0913
    ip_address,
    droplet_name,
    access_token,
    # transmission_port,
):
    "Open an SSH session to the droplet."

    if ip_address is None:
        try:
            ip_address = find_droplet_by_name(droplet_name, access_token).ip_address
        except DropletNotFound:
            click.secho("Droplet not found.", fg="cyan", err=True)
            sys.exit(1)
    run(f"ssh root@{ip_address}", shell=True, check=False)


@click.command("ssh-tunnel")
@click.option("-a", "--ip-address")
@click.option("-n", "--droplet-name", default="downloader")
@click.option("-T", "--access-token", type=str, envvar=ACCESS_TOKEN, show_envvar=True)
@click.option("-p", "--transmission-port", default=9091)
@conf_option(app_name="torrentdoer")
def ssh_tunnel(  # pylint: disable=R0913
    ip_address,
    droplet_name,
    access_token,
    transmission_port,
):
    "Open an SSH session to the droplet."

    if ip_address is None:
        try:
            ip_address = find_droplet_by_name(droplet_name, access_token).ip_address
        except DropletNotFound:
            click.secho("Droplet not found.", fg="cyan", err=True)
            sys.exit(1)
    make_tunnel(ip_address, transmission_port)



"Destroy droplet"
import sys

import click
from click_conf_file import conf_option

from .constants import ACCESS_TOKEN
from .droplet_helper import DropletNotFound, find_droplet_by_name


@click.command("destroy")
@click.option("-n", "--droplet-name", default="downloader")
@click.option("-t", "--access-token", type=str, envvar=ACCESS_TOKEN, show_envvar=True)
@conf_option(app_name="torrentdoer")
def destroy_droplet(droplet_name, access_token):
    "Destroy downloader."
    try:
        find_droplet_by_name(droplet_name, access_token).destroy()
        click.secho("Destroyed droplet.", fg="magenta", err=True)
    except DropletNotFound:
        click.secho("Droplet not found.", err=True, fg="red")
        sys.exit(1)
